import json

from django.templatetags.static import static
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe

from wagtail import hooks
from wagtail.snippets.models import register_snippet

from wagtaildraftsharing.actions import (
    register_wagtaildraftsharing_log_actions,
)
from wagtaildraftsharing.snippets import WagtaildraftsharingLinkSnippet


register_snippet(WagtaildraftsharingLinkSnippet)

hooks.register("register_log_actions")(
    register_wagtaildraftsharing_log_actions
)


@hooks.register("insert_global_admin_js")
def editor_js():
    return (
        format_html(
            """
        <script src="{}"></script>
        """,
            static("wagtaildraftsharing/js/wagtaildraftsharing.js"),
        )
        + mark_safe(
            f"""
        <script id="wagtaildraftsharing-config" type="application/json">
            {json.dumps({
                'urls': {
                    'create': reverse('wagtaildraftsharing:create'),
                },
            })}
        </script>"""
        )
    )



from wagtail.snippets.views.snippets import SnippetViewSet

from wagtaildraftsharing.models import WagtaildraftsharingLink


class WagtaildraftsharingLinkSnippet(SnippetViewSet):
    model = WagtaildraftsharingLink
    base_url_path = "wagtaildraftsharing"
    menu_icon = "pilcrow"
    menu_order = 200
    add_to_settings_menu = False
    add_to_admin_menu = True
    list_display = ("__str__", "is_active", "created_by", "share_url")
    list_filter = ("is_active",)

    def get_queryset(self, request):
        return WagtaildraftsharingLink.objects.all().prefetch_related(
            "revision", "revision__content_object", "created_by"
        )



import uuid

from django.conf import settings
from django.db import models
from django.urls import reverse
from django.utils.html import format_html


class WagtaildraftsharingLink(models.Model):
    key = models.UUIDField(
        unique=True, default=uuid.uuid4, editable=False, primary_key=False
    )
    revision = models.OneToOneField(
        "wagtailcore.Revision",
        on_delete=models.CASCADE,
        related_name="+",
    )
    is_active = models.BooleanField(default=True)
    active_until = models.DateTimeField(
        null=True, blank=True, help_text="Leave blank for no expiry"
    )
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        editable=False,
    )

    def __str__(self):
        return f"Revision {self.revision_id} of {self.revision.content_object}"

    class Meta:
        verbose_name = "Draftsharing Link"
        verbose_name_plural = "Draftsharing Links"

    @property
    def url(self):
        return reverse("wagtaildraftsharing:view", kwargs={"key": self.key})

    @property
    def share_url(self):
        return format_html(
            """<a data-wagtaildraftsharing-url
            class="button button-secondary button-small"
            target="_blank" rel="noopener noreferrer" href="{}">View</a>""",
            self.url,
        )



import json

from django.contrib.auth import get_user_model
from django.test import RequestFactory, TestCase

from wagtail_factories import PageFactory

from wagtaildraftsharing.models import WagtaildraftsharingLink
from wagtaildraftsharing.views import CreateSharingLinkView, SharingLinkView


User = get_user_model()


class TestViews(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.superuser = User.objects.create_superuser(
            username="test", password="test"
        )
        cls.factory = RequestFactory()

    def create_revision(self):
        page = PageFactory()

        # create the first revision
        page.save_revision().publish()

        old_title = page.title
        new_title = f"New {old_title}"
        page.title = new_title

        # create the second revision with a new title
        page.save_revision().publish()

        page.refresh_from_db()
        earliest_revision = page.revisions.earliest("created_at")
        return earliest_revision

    def test_create_sharing_link_view(self):
        revision = self.create_revision()
        request = self.factory.post(
            "/create/",
            {
                "revision": revision.id,
            },
        )
        request.user = self.superuser

        response = CreateSharingLinkView.as_view()(request)
        self.assertEqual(response.status_code, 200)

        response_data = json.loads(response.content.decode("utf-8"))
        self.assertEqual(
            response_data["url"], WagtaildraftsharingLink.objects.get().url
        )

    def test_sharing_link_view(self):
        revision = self.create_revision()
        sharing_link = WagtaildraftsharingLink.objects.create(
            revision=revision, created_by=self.superuser
        )

        request = self.factory.get(f"/{sharing_link.key}/")
        response = SharingLinkView.as_view()(request, key=sharing_link.key)

        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response, f"<title>{revision.as_object().title}</title>"
        )



import uuid

from django.conf import settings
from django.http import Http404, JsonResponse
from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator
from django.utils.timezone import now as datetime_now
from django.utils.timezone import timedelta
from django.views.generic import CreateView

from wagtail.admin.auth import user_has_any_page_permission, user_passes_test
from wagtail.admin.views.generic.preview import PreviewRevision
from wagtail.log_actions import log
from wagtail.models import Page, Revision

from wagtaildraftsharing.actions import WAGTAILDRAFTSHARING_CREATE_SHARING_LINK
from wagtaildraftsharing.forms import CreateWagtaildraftsharingLinkForm
from wagtaildraftsharing.models import WagtaildraftsharingLink


DEFAULT_MAX_AGE = 7 * 24 * 60 * 60  # 7 days
max_age = getattr(settings, "WAGTAILDRAFTSHARING_MAX_AGE", DEFAULT_MAX_AGE)


class SharingLinkView(PreviewRevision):
    def setup(self, request, *args, **kwargs):
        key = kwargs.pop("key")
        now = datetime_now()

        sharing_link = get_object_or_404(
            WagtaildraftsharingLink, key=key, is_active=True
        )
        if sharing_link.active_until and sharing_link.active_until < now:
            sharing_link.is_active = False
            sharing_link.save(update_fields=["is_active"])
            raise Http404

        revision_id = sharing_link.revision_id
        page_id = get_object_or_404(Revision, id=revision_id).object_id
        return super().setup(request, *args, page_id, revision_id, **kwargs)

    def get_object(self):
        return get_object_or_404(Page, id=self.pk).specific


@method_decorator(
    user_passes_test(user_has_any_page_permission), name="dispatch"
)
class CreateSharingLinkView(CreateView):
    model = WagtaildraftsharingLink
    form_class = CreateWagtaildraftsharingLinkForm

    def form_valid(self, form):
        revision = form.cleaned_data["revision"]
        key = uuid.uuid4()
        if max_age > 0:
            active_until = datetime_now() + timedelta(seconds=max_age)
        else:
            active_until = None
        sharing_link, created = WagtaildraftsharingLink.objects.get_or_create(
            revision=revision,
            defaults={
                "key": key,
                "created_by": self.request.user,
                "active_until": active_until,
            },
        )
        if created:
            log(
                instance=revision.content_object,
                action=WAGTAILDRAFTSHARING_CREATE_SHARING_LINK,
                user=self.request.user,
                revision=revision,
                data={"revision": revision.id},
            )
        return JsonResponse({"url": sharing_link.url})

    def form_invalid(self, form):
        return JsonResponse({"errors": form.errors}, status=400)



from django.utils.translation import gettext_lazy as _

from wagtail.log_actions import LogFormatter


WAGTAILDRAFTSHARING_CREATE_SHARING_LINK = (
    "wagtaildraftsharing.create_sharing_link"
)


def register_wagtaildraftsharing_log_actions(actions):
    @actions.register_action(WAGTAILDRAFTSHARING_CREATE_SHARING_LINK)
    class CreateSharingLink(LogFormatter):
        label = _("Create sharing link")

        def format_message(self, log_entry):
            return _("Created sharing link for revision {revision_id}").format(
                revision_id=log_entry.data["revision"],
                username=log_entry.user,
            )
